#include <iostream>
#include "AudioFile.hxx"
#include "MultiChannelAudioFileReader.hxx"
#include "Audio.hxx"
#include "CLAM_Math.hxx"
#include "DataTypes.hxx"
#include "Spectrum.hxx"
#include "SpectrumConfig.hxx"
#include "FFT_rfftw.hxx"
#include "XMLStorage.hxx"
#include "Err.hxx"
#include <FL/Fl.H>
#include "Fl_SMS_Spectrum.hxx"
#include <FL/Fl_Window.H>
#include "Array.hxx"


#include <vector>
int main( int argc, char** argv )
{
	if ( argc == 1 )
	{
		std::cerr << "No input file" << std::endl;
		std::cerr << "Usage: AudioFileReading <input file name>" << std::endl;
		exit( - 1 );
	}
	else if ( argc > 2 )
	{
		std::cerr << "Too many parameters" << std::endl;
		std::cerr << "Usage: AudioFileReading <input file name>" << std::endl;

		exit( -1 );
	}

	CLAM::AudioFile file;
	file.SetLocation( argv[1] );
	if ( !file.IsReadable() )
	{
		std::cerr << "Error: file " << file.GetLocation() << " cannot be opened ";
		std::cerr << "or is encoded in an unrecognized format" << std::endl;
		exit(-1);
	}

	const CLAM::TSize readSize = 1024;
	std::vector<CLAM::Audio> outputs;
	outputs.resize( file.GetHeader().GetChannels() );
	for ( unsigned i = 0; i < outputs.size(); i++ )
	{
		outputs[i].SetSize( readSize );
	}

	CLAM::MultiChannelAudioFileReaderConfig cfg;
	cfg.SetSourceFile( file );
	CLAM::MultiChannelAudioFileReader reader;
	reader.Configure( cfg );

	for( unsigned i = 0; i < outputs.size(); i++ )
	{
		reader.GetOutPorts().GetByNumber(i).Attach( outputs[i] );
	}
	const CLAM::TSize samples=1024; // the number of samples in the audio signal
	const CLAM::TData sampleRate=48000.0; // the sample rate of the audio signal

	CLAM::Spectrum myspectrum;
	CLAM::SpecTypeFlags specFlags;
	specFlags.bComplex=0;
	specFlags.bMagPhase = 1;
	specFlags.bMagPhaseBPF = 0;
	specFlags.bPolar = 0;
	myspectrum.SetType(specFlags);
	myspectrum.SetScale(CLAM::EScale::eLog);
	myspectrum.SetSize(readSize/2+1);

	// MRJ: You need to keep as many spectrums as channels - if you want to
	// display all channel fft results at the same time
	std::vector< CLAM::Spectrum > spectrums;

	for( unsigned i = 0; i < outputs.size(); i++ )
		spectrums.push_back( myspectrum );

	CLAM::FFTConfig fconfig;
	fconfig.SetAudioSize(readSize);
	CLAM::FFT_rfftw myfft;
	myfft.Configure( fconfig );

	std::vector<CLAMVM::Fl_SMS_Spectrum* > specViews;

	Fl_Window* topLevel = new Fl_Window( 100, 100, 640, 480 );
	{
		// MRJ: You need as many spec views as channels you
		// want to show at once
		for ( unsigned i = 0; i < outputs.size(); i++ )
		{
			
			CLAMVM::Fl_SMS_Spectrum* specWidget = new CLAMVM::Fl_SMS_Spectrum(
				0,0,640,480 );
			
			specViews.push_back(specWidget);
			
			specWidget ->SetYRange(0,50);
		}
	}
	topLevel->end();
	topLevel->show();

	// we start it
	myfft.Start();
	// Now we can safely Start() the processing
	reader.Start();

	// MRJ: use this variable to control the rate of refresh -
	// 30 means 30 frames per second
	double fps = 30.;

	CLAM::TIndex frameCount = 1;
	while ( reader.Do() )
	{
		for ( int i = 0; i < outputs.size(); i++ )
		{
			myfft.Do( outputs[i], spectrums[i] );
			specViews[i]->SetSpectrum( spectrums[i].GetMagBuffer(),
						   spectrums[i].GetSpectralRange() );
		}
	       
		// MRJ: Here we tell FLTK to refresh the widget fps times every
		// second
		Fl::wait( 1./fps );
		// MRJ: I have noticed that sometimes it is required to flush
		// FLTK own event queue if the FLTK thread is writing to stdout
		// or stderr
		Fl::flush();

	}
	// And stop it:
	reader.Stop();

	// MRJ: No need to the destroy the spectrum 
	// views - FLTK should care about it

	delete topLevel;
	return 0;
}

